import {
  select,    // for selecting the elememts
  csv,      // parsing the data
  scaleLinear,   // for the linear scales  which maps a range of input values to a range of output values.
  scaleTime,    //for creating a time scale
  extent,     // returns minimum and maximum value
  axisLeft,   // to create a left oreinted y-axis
  axisBottom,   // to create an x-axis
  format,    //to format numbers, dates etc
  line,  // for crreating a line generator
  area,   // for generating the svg path
  curveBasis  //for creating a curve
} from 'd3';

//selecting the  svg element in DOM
const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// providing the format to the number to show on the Areachart
const xAxisTickFormat = number =>
    format('.3s')(number)

//Creating Area graph for the given data
const render = data => {
  
   //Declaring constants and assiging values to them
  const xValue = d => d.Year;  //depicting years on x axis
  const xAxisLabel = 'Year';
  
  const yValue = d => d.IndiaGDP;  //depicting values on y axis
  const circleRadius = 7;
  const yAxisLabel = 'IndiaGDP';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;  
  
// Providing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
// Providing the Scale for Y axis  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
 // declaring the Position for the svg element   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
 // Assigning values to x axis 
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
 // Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
  .tickFormat(xAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  //Adding yaxis label to Area chart
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  //Adding xaxis label to Area chart
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
  
 //  drawing the line on the curve and adding it to svg 
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
 // drawing the area under the curve and adding it to svg 
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  //creating a line chart
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  
  //creating area chart
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};

// Loading and Parsing CSV data that we have given
csv('https://gist.githubusercontent.com/prachow/a02d466980723b8e6269f233a946c2ca/raw/b535ae16cd5b65d902f8f901131959889e5da583/indiagdp.csv')
  .then(data => {
    data.forEach(d => {
      d.value = +d.value;
//Formatting to retrieve year from the date
      
    	d.year = +d.year;
      //d.year = d3.timeParse("%d/%m/%Y")(d.year);
    });
    render(data);
  	console.log(data);
  });